@@ -174,13 +174,19 @@ class _EpydocLinker(object):
         if target is not None:
             return self._objLink(target, prettyID)
 
-        target = self.look_for_intersphinx(fullID)
+        target = self.look_for_intersphinx(fullerID)
+        if not target:
+            # FIXME: https://github.com/twisted/pydoctor/issues/125
+            # expandName is unreliable so in the case fullerID fails, we
+            # try our luck with fullID.
+            target = self.look_for_intersphinx(fullID)
         if target:
             return '<a href="%s"><code>%s</code></a>'%(target, prettyID)
 
         self.obj.system.msg(
-            "translate_identifier_xref", "%s:%s invalid ref to %s" % (
-                self.obj.fullName(), self.obj.linenumber, fullID),
+            "translate_identifier_xref", "%s:%s invalid ref to '%s' "
+            "resolved as '%s'" % (
+                self.obj.fullName(), self.obj.linenumber, fullID, fullerID),
             thresh=-1)
         return '<code>%s</code>'%(prettyID,)
 
