@@ -14,6 +14,8 @@ import sys
 import types
 import __builtin__
 
+from pydoctor.sphinx import SphinxInventory
+
 # originally when I started to write pydoctor I had this idea of a big
 # tree of Documentables arranged in an almost arbitrary tree.
 #
@@ -23,7 +25,8 @@ import __builtin__
 #
 #   Packages can contain Packages and Modules
 #   Modules can contain Functions and Classes
-#   Classes can contain Functions (when they get called Methods) and Classes
+#   Classes can contain Functions (in this case they get called Methods) and
+#       Classes
 #   Functions can't contain anything.
 
 
@@ -41,7 +44,6 @@ class Documentable(object):
 
     @ivar docstring: The object's docstring.  But also see docsources.
     @ivar system: The system the object is part of.
-    @ivar prefix: ...
     @ivar parent: ...
     @ivar parentMod: ...
     @ivar name: ...
@@ -62,15 +64,6 @@ class Documentable(object):
 
     def __init__(self, system, name, docstring, parent=None):
         self.system = system
-        if parent is not None:
-            if (parent.parent and isinstance(parent.parent, Package)
-                and isinstance(parent, Module)
-                and parent.name == '__init__'):
-                self.prefix = parent.parent.fullName() + '.'
-            else:
-                self.prefix = parent.fullName() + '.'
-        else:
-            self.prefix = ''
         self.name = name
         self.docstring = docstring
         self.parent = parent
@@ -82,10 +75,19 @@ class Documentable(object):
     def setup(self):
         self.contents = {}
         self.orderedcontents = []
-        self._name2fullname = {}
 
     def fullName(self):
-        return self.prefix + self.name
+        parent = self.parent
+        if parent is not None:
+            if (parent.parent and isinstance(parent.parent, Package)
+                and isinstance(parent, Module)
+                and parent.name == '__init__'):
+                prefix = parent.parent.fullName() + '.'
+            else:
+                prefix = parent.fullName() + '.'
+        else:
+            prefix = ''
+        return prefix + self.name
 
     def __repr__(self):
         return "%s %r"%(self.__class__.__name__, self.fullName())
@@ -99,84 +101,73 @@ class Documentable(object):
         """
         yield self
 
-    def name2fullname(self, name):
-        """XXX what is the difference between name2fullname,
-        dottedNameToFullName and resolveDottedName??"""
-        if name in self._name2fullname:
-            return self._name2fullname[name]
-        else:
-            return self.parent.name2fullname(name)
 
-    def _resolveName(self, name, verbose):
-        """Helper for resolveDottedName."""
-        system = self.system
-        obj = self
-        while obj:
-            if name in obj.contents:
-                return obj.contents[name]
-            elif name in obj._name2fullname:
-                fn = obj._name2fullname[name]
-                o = system.allobjects.get(fn)
-                if o is None:
-                    for othersys in system.moresystems:
-                        o = othersys.allobjects.get(fn)
-                        if o is not None:
-                            break
-                if o is None and verbose > 0:
-                    print "from %r, %r resolves to %r which isn't present in the system"%(
-                        self.fullName(), name, fn)
-                return o
-            obj = obj.parent
-        obj = self
-        while obj:
-            for n, fn in obj._name2fullname.iteritems():
-                o2 = system.allobjects.get(fn)
-                if o2 and name in o2.contents:
-                    return o2.contents[name]
-            obj = obj.parent
-        if name in system.allobjects:
-            return system.allobjects[name]
-        for othersys in system.moresystems:
-            if name in othersys.allobjects:
-                return othersys.allobjects[name]
-        if verbose > 0:
-            print "failed to find %r from %r"%(name, self.fullName())
-        return None
+    def reparent(self, new_parent, new_name):
+        # this code attempts to preserve "rather a lot" of
+        # invariants assumed by various bits of pydoctor
+        # and that are of course not written down anywhere
+        # :/
+        self._handle_reparenting_pre()
+        old_parent = self.parent
+        old_name = self.name
+        self.parent = self.parentMod = new_parent
+        self.name = new_name
+        self._handle_reparenting_post()
+        del old_parent.contents[old_name]
+        old_parent.orderedcontents.remove(self)
+        old_parent._localNameToFullName_map[old_name] = self.fullName()
+        new_parent.contents[new_name] = self
+        new_parent.orderedcontents.append(self)
+        self._handle_reparenting_post()
 
-    def resolveDottedName(self, dottedname, verbose=None):
-        """XXX what is the difference between name2fullname,
-        dottedNameToFullName and resolveDottedName??"""
-        if verbose is None:
-            verbose = self.system.options.verbosity
-        parts = dottedname.split('.')
-        obj = self._resolveName(parts[0], verbose)
-        if obj is None:
-            return obj
-        for p in parts[1:]:
-            if p not in obj._name2fullname:
-                return None
-            fn = obj._name2fullname[p]
-            if fn not in self.system.allobjects:
-                return None
-            obj = self.system.allobjects[fn]
-        if verbose > 1:
-            print dottedname, '->', obj.fullName(), 'in', self.fullName()
-        return obj
+    def _handle_reparenting_pre(self):
+        del self.system.allobjects[self.fullName()]
+        for o in self.orderedcontents:
+            o._handle_reparenting_pre()
 
-    def dottedNameToFullName(self, dottedname):
-        """XXX what is the difference between name2fullname,
-        dottedNameToFullName and resolveDottedName??"""
-        if '.' not in dottedname:
-            start, rest = dottedname, ''
-        else:
-            start, rest = dottedname.split('.', 1)
-            rest = '.' + rest
+    def _handle_reparenting_post(self):
+        self.system.allobjects[self.fullName()] = self
+        for o in self.orderedcontents:
+            o._handle_reparenting_post()
+
+    def _localNameToFullName(self, name):
+        raise NotImplementedError(self._localNameToFullName)
+
+    def expandName(self, name):
+        """Return a fully qualified name for the possibly-dotted `name`.
+
+        To explain what this means, consider the following modules:
+
+        mod1.py::
+
+            from external_location import External
+            class Local(object):
+                pass
+
+        mod2.py::
+
+            from mod1 import External as RenamedExternal
+            import mod1 as renamed_mod
+            class E:
+                pass
+
+        In the context of mod2.E, expandName("RenamedExternal") should be
+        "external_location.External" and expandName("renamed_mod.Local")
+        should be "mod1.Local". """
+        parts = name.split('.')
         obj = self
-        while start not in obj._name2fullname:
-            obj = obj.parent
-            if obj is None or isinstance(obj, Package):
-                return dottedname
-        return obj._name2fullname[start] + rest
+        for i, p in enumerate(parts):
+            full_name = obj._localNameToFullName(p)
+            obj = self.system.objForFullName(full_name)
+            if obj is None:
+                break
+        remaning = parts[i+1:]
+        return '.'.join([full_name] + remaning)
+
+    def resolveName(self, name):
+        """Return the object named by "name" (using Python's lookup rules) in
+        this context, if any is known to pydoctor."""
+        return self.system.objForFullName(self.expandName(name))
 
     @property
     def privacyClass(self):
@@ -231,48 +222,74 @@ class Documentable(object):
 
 class Package(Documentable):
     kind = "Package"
-    def name2fullname(self, name):
-        raise NameError
     def docsources(self):
         yield self.contents['__init__']
     @property
     def doctarget(self):
         return self.contents['__init__']
+    @property
+    def state(self):
+        return self.contents['__init__'].state
+
+    def _localNameToFullName(self, name):
+        if name in self.contents:
+            o = self.contents[name]
+            return o.fullName()
+        else:
+            return self.contents['__init__']._localNameToFullName(name)
+
 
 [UNPROCESSED, PROCESSING, PROCESSED] = range(3)
 
-class Module(Documentable):
+class CanContainImportsDocumentable(Documentable):
+    def setup(self):
+        super(CanContainImportsDocumentable, self).setup()
+        self._localNameToFullName_map = {}
+
+
+class Module(CanContainImportsDocumentable):
     kind = "Module"
     state = UNPROCESSED
     linenumber = 0
     def setup(self):
         super(Module, self).setup()
         self.all = None
-    def name2fullname(self, name):
-        if name in self._name2fullname:
-            return self._name2fullname[name]
+
+    def _localNameToFullName(self, name):
+        if name in self.contents:
+            o = self.contents[name]
+            return o.fullName()
+        elif name in self._localNameToFullName_map:
+            return self._localNameToFullName_map[name]
         elif name in __builtin__.__dict__:
             return name
         else:
-            self.system.warning("optimistic name resolution", name)
             return name
 
 
-class Class(Documentable):
+class Class(CanContainImportsDocumentable):
     kind = "Class"
     def setup(self):
         super(Class, self).setup()
-        self.bases = []
         self.rawbases = []
-        self.baseobjects = []
         self.subclasses = []
-    def allbases(self):
+
+    def allbases(self, include_self=False):
+        if include_self:
+            yield self
         for b in self.baseobjects:
             if b is None:
                 continue
-            yield b
-            for b2 in b.allbases():
+            for b2 in b.allbases(True):
                 yield b2
+    def _localNameToFullName(self, name):
+        if name in self.contents:
+            o = self.contents[name]
+            return o.fullName()
+        elif name in self._localNameToFullName_map:
+            return self._localNameToFullName_map[name]
+        else:
+            return self.parent._localNameToFullName(name)
 
 
 class Function(Documentable):
@@ -287,10 +304,11 @@ class Function(Documentable):
         yield self
         if not isinstance(self.parent, Class):
             return
-        for b in self.parent.allbases():
+        for b in self.parent.allbases(include_self=False):
             if self.name in b.contents:
                 yield b.contents[self.name]
-
+    def _localNameToFullName(self, name):
+        return self.parent._localNameToFullName(name)
 
 class Attribute(Documentable):
 
@@ -298,6 +316,8 @@ class Attribute(Documentable):
     kind = "Attribute"
     documentation_location = DocLocation.PARENT_PAGE
 
+    def _localNameToFullName(self, name):
+        return self.parent._localNameToFullName(name)
 
 class PrivacyClass:
     """'enum' containing values indicating how private an object should be.
@@ -329,7 +349,7 @@ class System(object):
     #defaultBuilder = astbuilder.ASTBuilder
     sourcebase = None
 
-    def __init__(self):
+    def __init__(self, options=None):
         self.allobjects = {}
         self.orderedallobjects = []
         self.rootobjects = []
@@ -338,11 +358,16 @@ class System(object):
         self.moresystems = []
         self.subsystems = []
         self.urlprefix = ''
-        from pydoctor.driver import parse_args
-        self.options, _ = parse_args([])
-        self.options.verbosity = 3
+
+        if options:
+            self.options = options
+        else:
+            from pydoctor.driver import parse_args
+            self.options, _ = parse_args([])
+            self.options.verbosity = 3
+
         self.abbrevmapping = {}
-        self.guessedprojectname = 'my project'
+        self.projectname = 'my project'
         self.epytextproblems = [] # fullNames of objects that failed to epytext properly
         self.verboselevel = 0
         self.needsnl = False
@@ -351,6 +376,10 @@ class System(object):
         self.module_count = 0
         self.processing_modules = []
         self.buildtime = datetime.datetime.now()
+        # Once pickle support is removed, System should be
+        # initialized with project name so that we can reuse intersphinx instance for
+        # object.inv generation.
+        self.intersphinx = SphinxInventory(logger=self.msg, project_name=self.projectname)
 
     def verbosity(self, section=None):
         if isinstance(section, str):
@@ -389,39 +418,10 @@ class System(object):
                 self.needsnl = False
                 print
 
-    def resolveAlias(self, n):
-        if '.' not in n:
-            return n
-        mod, clsname = n.rsplit('.', 1)
-        if not mod:
-            return mod
-        systems = [self] + self.moresystems
-        for system in systems:
-            if mod in system.allobjects:
-                break
-        else:
-            return n
-        m = system.allobjects[mod]
-        if isinstance(m, Package):
-            m = m.contents['__init__']
-        if not isinstance(m, Module):
-            return n
-        if clsname in m._name2fullname:
-            newname = m.name2fullname(clsname)
-            if newname == n:
-                return newname
-            for system in systems:
-                if newname in system.allobjects:
-                    return newname
-            else:
-                return self.resolveAlias(newname)
-        else:
-            return n
-
     def objForFullName(self, fullName):
-        for sys in [self] + self.moresystems:
-            if fullName in sys.allobjects:
-                return sys.allobjects[fullName]
+        for system in [self] + self.moresystems:
+            if fullName in system.allobjects:
+                return system.allobjects[fullName]
         return None
 
     def _warning(self, current, type, detail):
@@ -445,22 +445,27 @@ class System(object):
             return PrivacyClass.PRIVATE
         return PrivacyClass.VISIBLE
 
+    def __getstate__(self):
+        d = self.__dict__.copy()
+        del d['intersphinx']
+        return d
+
     def __setstate__(self, state):
         if 'abbrevmapping' not in state:
             state['abbrevmapping'] = {}
         # this is so very, very evil.
         # see doc/extreme-pickling-pain.txt for more.
         def lookup(name):
-            for sys in [self] + self.moresystems + self.subsystems:
-                if name in sys.allobjects:
-                    return sys.allobjects[name]
+            for system in [self] + self.moresystems + self.subsystems:
+                if name in system.allobjects:
+                    return system.allobjects[name]
             raise KeyError, name
         self.__dict__.update(state)
-        for sys in [self] + self.moresystems + self.subsystems:
-            if 'allobjects' not in sys.__dict__:
+        for system in [self] + self.moresystems + self.subsystems:
+            if 'allobjects' not in system.__dict__:
                 return
-        for sys in [self] + self.moresystems + self.subsystems:
-            for obj in sys.orderedallobjects:
+        for system in [self] + self.moresystems + self.subsystems:
+            for obj in system.orderedallobjects:
                 for k, v in obj.__dict__.copy().iteritems():
                     if k.startswith('$'):
                         del obj.__dict__[k]
@@ -480,13 +485,13 @@ class System(object):
                             n[kk] = lookup(vv)
                         del obj.__dict__[k]
                         obj.__dict__[k[1:]] = n
+        self.intersphinx = SphinxInventory(logger=self.msg, project_name=self.projectname)
 
     def addObject(self, obj):
         """Add C{object} to the system."""
         if obj.parent:
             obj.parent.orderedcontents.append(obj)
             obj.parent.contents[obj.name] = obj
-            obj.parent._name2fullname[obj.name] = obj.fullName()
         else:
             self.rootobjects.append(obj)
         self.orderedallobjects.append(obj)
@@ -587,6 +592,9 @@ class System(object):
                 self.addObject(f)
             elif isinstance(v, type):
                 c = self.Class(self, k, v.__doc__, parent)
+                c.bases = []
+                c.baseobjects = []
+                c.rawbases = []
                 c.parentMod = parentMod
                 self.addObject(c)
                 self._introspectThing(v, c, parentMod)
@@ -662,9 +670,18 @@ class System(object):
         while (fn + ' ' + str(i)) in self.allobjects:
             i += 1
         prev = self.allobjects[obj.fullName()]
+        self._warning(obj.parent, "duplicate", prev)
+        def remove(o):
+            del self.allobjects[o.fullName()]
+            for c in o.orderedcontents:
+                remove(c)
+        remove(prev)
         prev.name = obj.name + ' ' + str(i)
-        self.allobjects[prev.fullName()] = prev
-        self._warning(obj.parent, "duplicate", self.allobjects[obj.fullName()])
+        def readd(o):
+            self.allobjects[o.fullName()] = o
+            for c in o.orderedcontents:
+                readd(c)
+        readd(prev)
         self.allobjects[obj.fullName()] = obj
         return obj
 
@@ -674,7 +691,7 @@ class System(object):
         if mod is None:
             return None
         if isinstance(mod, Package):
-            return self.getProcessedModule(modname + '.__init__')
+            return self.getProcessedModule(modname + '.__init__').parent
         if not isinstance(mod, Module):
             return None
 
@@ -691,7 +708,7 @@ class System(object):
             return
         builder = self.defaultBuilder(self)
         ast = builder.parseFile(mod.filepath)
-        if  ast:
+        if ast:
             self.processing_modules.append(mod.fullName())
             self.msg("processModule", "processing %s"%(self.processing_modules), 1)
             builder.processModuleAST(ast, mod)
@@ -711,3 +728,11 @@ class System(object):
         while self.unprocessed_modules:
             mod = iter(self.unprocessed_modules).next()
             self.processModule(mod)
+
+
+    def fetchIntersphinxInventories(self):
+        """
+        Download and parse intersphinx inventories based on configuration.
+        """
+        for url in self.options.intersphinx:
+            self.intersphinx.update(url)
