@@ -4,8 +4,7 @@ from twisted.web.template import tags, Element, renderer, XMLFile
 
 from pydoctor import epydoc2stan, model
 from pydoctor.templatewriter.pages.table import ChildTable
-from pydoctor.templatewriter.util import \
-     templatefile, srclink, taglink
+from pydoctor.templatewriter import util
 
 def getBetterThanArgspec(argspec):
     """Ok, maybe argspec's format isn't the best after all: This takes an
@@ -52,33 +51,28 @@ class DocGetter(object):
         return epydoc2stan.doc2stan(ob, summary=summary)[0]
 
 class CommonPage(Element):
-    loader = XMLFile(templatefile('common.html'))
 
     def __init__(self, ob, docgetter=None):
         self.ob = ob
         if docgetter is None:
             docgetter = DocGetter()
         self.docgetter = docgetter
-        self.usesorttable = ob.system.options.htmlusesorttable
-        self.usesplitlinks = ob.system.options.htmlusesplitlinks
-        self.shortenlists = ob.system.options.htmlshortenlists
+
+    @property
+    def loader(self):
+        return XMLFile(util.templatefile('common.html'))
 
     def title(self):
         return self.ob.fullName()
 
     def mediumName(self, obj):
-        fn = obj.fullName()
-        if '.' not in fn:
-            return fn
-        path, name = fn.rsplit('.', 1)
-        def process(part):
-            return obj.system.abbrevmapping.get(part, part[0])
-        return '.'.join([process(p) for p in path.split('.')]) + '.' + name
+        return self.ob.fullName()
 
     def heading(self):
         return tags.h1(class_=self.ob.css_class)(
-            self.mediumName(self.ob), " : ", self.ob.kind.lower(),
-            " documentation")
+            tags.code(self.mediumName(self.ob)), " ",
+            tags.small(self.ob.kind.lower(), " documentation"),
+            )
 
     def part(self):
         if self.ob.parent:
@@ -87,26 +81,31 @@ class CommonPage(Element):
                 parent = parent.parent
             parts = []
             while parent.parent:
-                parts.append(taglink(parent, parent.name))
+                parts.append(util.taglink(parent, parent.name))
                 parts.append('.')
                 parent = parent.parent
-            parts.append(taglink(parent, parent.name))
+            parts.append(util.taglink(parent, parent.name))
             parts.reverse()
-            return 'Part of ', parts
+            return 'Part of ', tags.code(parts)
         else:
             return []
 
     def project(self):
         if self.ob.system.options.projecturl:
-            return tags.a(href=self.ob.system.options.projecturl)(self.ob.system.options.projectname)
-        elif self.ob.system.options.projectname:
-            return self.ob.system.options.projectname
+            return tags.a(href=self.ob.system.options.projecturl)(self.ob.system.projectname)
         else:
-            return self.ob.system.guessedprojectname
+            return self.ob.system.projectname
+
+    @renderer
+    def deprecated(self, request, tag):
+        if hasattr(self.ob, "_deprecated_info"):
+            return (tags.div(self.ob._deprecated_info, role="alert", class_="deprecationNotice alert alert-warning"),)
+        else:
+            return ()
 
     @renderer
     def source(self, request, tag):
-        sourceHref = srclink(self.ob)
+        sourceHref = util.srclink(self.ob)
         if not sourceHref:
             return ()
         return tag(href=sourceHref)
@@ -133,26 +132,10 @@ class CommonPage(Element):
     def baseTables(self, request, tag):
         return ()
 
-    @renderer
-    def bigTable(self, request, tag):
-        return ()
-
     def mainTable(self):
         children = self.children()
         if children:
-            return ChildTable(self.docgetter, self.ob, self.has_lineno_col(), children)
-        else:
-            return ()
-
-    def has_lineno_col(self):
-        if not self.usesorttable:
-            return False
-        return isinstance(self.ob, (model.Class, model.Module))
-
-    @renderer
-    def ifusesorttable(self, request, tag):
-        if self.usesorttable:
-            return tag
+            return ChildTable(self.docgetter, self.ob, children)
         else:
             return ()
 
@@ -178,17 +161,6 @@ class CommonPage(Element):
     def functionBody(self, data):
         return self.docgetter.get(data)
 
-    @renderer
-    def splittingLinks(self, request, tag):
-        return ()
-
-    @renderer
-    def pydoctorjs(self, request, tag):
-        if self.usesplitlinks or self.shortenlists:
-            return tag
-        else:
-            return ()
-
     @renderer
     def all(self, request, tag):
         return tag.fillSlots(
@@ -216,8 +188,9 @@ class PackagePage(CommonPage):
             [o for o in init.orderedcontents if o.isVisible],
             key=lambda o2:(-o2.privacyClass, o2.fullName()))
         if children:
-            return [tags.p("From the __init__.py module:"),
-                    ChildTable(self.docgetter, init, self.usesorttable, children)]
+            return [tags.p("From the ", tags.code("__init__.py"), " module:",
+                           class_="fromInitPy"),
+                    ChildTable(self.docgetter, init, children)]
         else:
             return ()
 
@@ -254,7 +227,7 @@ def unmasked_attrs(baselist):
     return [o for o in baselist[0].orderedcontents if o.name not in maybe_masking]
 
 
-def maybeShortenList(system, label, lst, idbase):
+def assembleList(system, label, lst, idbase):
     lst2 = []
     for name in lst:
         o = system.allobjects.get(name)
@@ -265,7 +238,7 @@ def maybeShortenList(system, label, lst, idbase):
         return None
     def one(item):
         if item in system.allobjects:
-            return taglink(system.allobjects[item])
+            return util.taglink(system.allobjects[item])
         else:
             return item
     def commasep(items):
@@ -276,25 +249,7 @@ def maybeShortenList(system, label, lst, idbase):
         del r[-1]
         return r
     p = [label]
-    if len(lst) <= 5 or not system.options.htmlshortenlists:
-        p.extend(commasep(lst))
-    else:
-        p.extend(commasep(lst[:3]))
-        q = [', ']
-        q.extend(commasep(lst[3:]))
-        q.append(tags.span(class_='showIfJS')[
-            ' ',
-            tags.a(href="#",
-                   onclick="showAndHide('%s');"%idbase,
-                   class_="jslink")
-            ['(hide last %d again)'%len(lst[3:])]])
-        p.append(tags.span(id=idbase, class_='hideIfJS')[q])
-        p.append(tags.span(id=idbase+'Link', class_='showIfJS')[
-            ' ',
-            tags.a(href="#",
-                   onclick="hideAndShow('%s');"%idbase,
-                   class_="jslink")
-            ['... and %d more'%len(lst[3:])]])
+    p.extend(commasep(lst))
     return p
 
 
@@ -302,7 +257,6 @@ class ClassPage(CommonPage):
     def __init__(self, ob, docgetter=None):
         CommonPage.__init__(self, ob, docgetter)
         self.baselists = []
-        self.usesplitlinks = ob.system.options.htmlusesplitlinks
         for baselist in nested_bases(self.ob):
             attrs = unmasked_attrs(baselist)
             if attrs:
@@ -314,19 +268,12 @@ class ClassPage(CommonPage):
         scs = sorted(self.ob.subclasses, key=lambda o:o.fullName().lower())
         if not scs:
             return r
-        p = maybeShortenList(self.ob.system, "Known subclasses: ",
-                             [o.fullName() for o in scs], "moreSubclasses")
+        p = assembleList(self.ob.system, "Known subclasses: ",
+                         [o.fullName() for o in scs], "moreSubclasses")
         if p is not None:
             r.append(tags.p(p))
         return r
 
-    @renderer
-    def splittingLinks(self, request, tag):
-        if self.usesplitlinks and len(self.baselists) > 1:
-            return tag
-        else:
-            return ()
-
     def mediumName(self, ob):
         r = [super(ClassPage, self).mediumName(ob)]
         zipped = zip(self.ob.rawbases, self.ob.bases, self.ob.baseobjects)
@@ -336,7 +283,7 @@ class ClassPage(CommonPage):
                 if o is None:
                     r.append(tags.span(title=m)(n))
                 else:
-                    r.append(taglink(o, n))
+                    r.append(util.taglink(o, n))
                 if i != len(zipped)-1:
                     r.append(', ')
             r.append(')')
@@ -355,48 +302,38 @@ class ClassPage(CommonPage):
             del baselists[0]
         return [item.fillSlots(
                           baseName=self.baseName(b),
-                          baseTable=ChildTable(self.docgetter, self.ob, self.has_lineno_col(),
-                                                  sorted(attrs, key=lambda o:-o.privacyClass)))
+                          baseTable=ChildTable(self.docgetter, self.ob,
+                                               sorted(attrs, key=lambda o:-o.privacyClass)))
                 for b, attrs in baselists]
 
     def baseName(self, data):
         r = []
         source_base = data[0]
-        r.append(taglink(source_base, source_base.name))
+        r.append(util.taglink(source_base, source_base.name))
         bases_to_mention = data[1:-1]
         if bases_to_mention:
             tail = []
             for b in reversed(bases_to_mention):
-                tail.append(taglink(b, b.name))
+                tail.append(util.taglink(b, b.name))
                 tail.append(', ')
             del tail[-1]
             r.extend([' (via ', tail, ')'])
         return r
 
-    @renderer
-    def bigTable(self, request, tag):
-        if not self.usesplitlinks or len(self.baselists) == 1:
-            return ()
-        all_attrs = []
-        for b, attrs in self.baselists:
-            all_attrs.extend(attrs)
-        all_attrs.sort(key=lambda o:(-o.privacyClass, o.name.lower()))
-        return tag(ChildTable(self.docgetter, self.ob, self.has_lineno_col(), all_attrs))
-
     def functionExtras(self, data):
         r = []
         for b in self.ob.allbases(include_self=False):
             if data.name not in b.contents:
                 continue
             overridden = b.contents[data.name]
-            r.append(tags.div(class_="interfaceinfo")('overrides ', taglink(overridden)))
+            r.append(tags.div(class_="interfaceinfo")('overrides ', util.taglink(overridden)))
             break
         ocs = sorted(overriding_subclasses(self.ob, data.name), key=lambda o:o.fullName().lower())
         if ocs:
             self.overridenInCount += 1
             idbase = 'overridenIn' + str(self.overridenInCount)
-            l = maybeShortenList(self.ob.system, 'overridden in ',
-                                 [o.fullName() for o in ocs], idbase)
+            l = assembleList(self.ob.system, 'overridden in ',
+                             [o.fullName() for o in ocs], idbase)
             if l is not None:
                 r.append(tags.div(class_="interfaceinfo")(l))
         return r
@@ -405,12 +342,6 @@ class ClassPage(CommonPage):
 class ZopeInterfaceClassPage(ClassPage):
     def extras(self):
         r = [super(ZopeInterfaceClassPage, self).extras()]
-        system = self.ob.system
-        def tl(s):
-            if s in system.allobjects:
-                return taglink(system.allobjects[s])
-            else:
-                return s
         if self.ob.isinterface:
             namelist = sorted([o.fullName() for o in self.ob.implementedby_directly], key=lambda x:x.lower())
             label = 'Known implementations: '
@@ -418,7 +349,7 @@ class ZopeInterfaceClassPage(ClassPage):
             namelist = sorted(self.ob.implements_directly, key=lambda x:x.lower())
             label = 'Implements interfaces: '
         if namelist:
-            l = maybeShortenList(self.ob.system, label, namelist, "moreInterface")
+            l = assembleList(self.ob.system, label, namelist, "moreInterface")
             if l is not None:
                 r.append(tags.p(l))
         return r
@@ -437,7 +368,7 @@ class ZopeInterfaceClassPage(ClassPage):
         imeth = self.interfaceMeth(data.name)
         r = []
         if imeth:
-            r.append(tags.div(class_="interfaceinfo")('from ', taglink(imeth, imeth.parent.fullName())))
+            r.append(tags.div(class_="interfaceinfo")('from ', util.taglink(imeth, imeth.parent.fullName())))
         r.extend(super(ZopeInterfaceClassPage, self).functionExtras(data))
         return r
 
