@@ -78,7 +78,7 @@ def test_class_with_base():
     '''
     mod = fromText(src)
     assert len(mod.contents) == 2
-    clsC, clsD = mod.contents.values()
+    clsC, clsD = mod.orderedcontents
     assert clsC.fullName() == '<test>.C'
     assert clsC.docstring == None
     assert len(clsC.contents) == 1
@@ -91,6 +91,17 @@ def test_class_with_base():
     base, = clsD.bases
     assert base == '<test>.C'
 
+def test_follow_renaming():
+    src = '''
+    class C: pass
+    D = C
+    class E(D): pass
+    '''
+    mod = fromText(src)
+    C = mod.contents['C']
+    E = mod.contents['E']
+    assert E.baseobjects == [C], E.baseobjects
+
 def test_class_with_base_from_module():
     src = '''
     from X.Y import A
@@ -129,9 +140,9 @@ def test_class_with_base_from_module():
 
     assert len(clsD.bases) == 3
     base1, base2, base3 = clsD.bases
-    assert base1 == 'X.A'
-    assert base2 == 'X.B.C'
-    assert base3 == 'Y.Z.C'
+    assert base1 == 'X.A', base1
+    assert base2 == 'X.B.C', base2
+    assert base3 == 'Y.Z.C', base3
 
 def test_aliasing():
     def addsrc(system):
@@ -153,11 +164,6 @@ def test_aliasing():
 
     system = model.System()
     addsrc(system)
-    assert system.allobjects['c.C'].bases == ['b.B']
-
-    system = model.System()
-    system.options.resolvealiases = True
-    addsrc(system)
     assert system.allobjects['c.C'].bases == ['a.A']
 
 def test_more_aliasing():
@@ -181,25 +187,22 @@ def test_more_aliasing():
         fromText(src_b, 'b', system)
         fromText(src_c, 'c', system)
         fromText(src_d, 'd', system)
-    system = model.System()
-    addsrc(system)
-    assert system.allobjects['d.D'].bases == ['c.C']
 
     system = model.System()
-    system.options.resolvealiases = True
     addsrc(system)
     assert system.allobjects['d.D'].bases == ['a.A']
 
 def test_aliasing_recursion():
     system = model.System()
-    system.options.resolvealiases = True
     src = '''
+    class C:
+        pass
     from mod import C
     class D(C):
         pass
     '''
     mod = fromText(src, 'mod', system)
-    assert mod.contents['D'].bases == ['mod.C']
+    assert mod.contents['D'].bases == ['mod.C'], mod.contents['D'].bases
 
 def test_subclasses():
     src = '''
@@ -230,7 +233,7 @@ def test_nested_class_inheriting_from_same_module():
         class C(A):
             pass
     '''
-    mod = fromText(src)
+    fromText(src)
 
 def test_all_recognition():
     mod = fromText('''
@@ -266,21 +269,6 @@ def test_classmethod():
     assert mod.contents['C'].contents['f'].kind == 'Class Method'
 
 def test_classdecorator():
-    mod = fromText('''
-    @cd
-    class C:
-        pass
-    ''')
-    assert mod.contents['C'].decorators == [(('cd', 'cd', None), None)], \
-      mod.contents['C'].decorators
-    mod = fromText('''
-    @module.cd
-    class C:
-        pass
-    ''')
-    assert mod.contents['C'].decorators == [(('module.cd', 'module.cd', None), None)], \
-      mod.contents['C'].decorators
-
     mod = fromText('''
     def cd(cls):
         pass
@@ -295,21 +283,25 @@ def test_classdecorator():
 
 def test_classdecorator_with_args():
     mod = fromText('''
-    @cd()
-    class C:
-        pass
-    ''', modname='test')
-    C = mod.contents['C']
-    assert C.decorators == [(('cd', 'cd', None), [])], \
-      C.decorators
-    mod = fromText('''
+    def cd(): pass
     class A: pass
     @cd(A)
     class C:
         pass
     ''', modname='test')
+    cd = mod.contents['cd']
     A = mod.contents['A']
     C = mod.contents['C']
-    assert C.decorators == [(('cd', 'cd', None), [('A', 'test.A', A)])], \
+    assert C.decorators == [(('cd', 'test.cd', cd), [('A', 'test.A', A)])], \
       C.decorators
 
+
+def test_import_star():
+    mod_a = fromText('''
+    def f(): pass
+    ''', modname='a')
+    mod_b = fromText('''
+    from a import *
+    ''', modname='b', system=mod_a.system)
+    assert mod_b.resolveName('f') == mod_a.contents['f']
+
